# -*- coding: utf-8 -*-

"""
Text class.
"""

from typing import List
from textarium.utils import pipeline
import textarium.preprocessing as tp

class Text:
    def __init__(self, text: str, lang: str = 'en'):
        self.lang = lang
        self.raw_text = text
        self.prepared_text = None

    def prepare(self, lemmatize: bool = True, stopwords: List[str] = None):
        """Prepares text for analysis and ML modeling:
            - Removes punctuation, digits and extra spaces
            - Removes stopwords, html-tags and urls
            - Lemmatizes all words

        Returns:
            str: A prepared string
        """
        text = pipeline(self.raw_text, 
            [
                tp.remove_html,
                tp.remove_urls,
                tp.remove_digits,
                tp.make_lowercase,
                tp.remove_punctuation,
            ]
        )
        if lemmatize:
            text = tp.lemmatize(text, lang=self.lang)
        if stopwords:
            text = tp.remove_words(text, words_to_exclude=stopwords)
        self.prepared_text = text




# -*- coding: utf-8 -*-

"""
Methods for calculating different statistics for a Text object.
"""
from typing import List, Union, Callable
from textarium import (
    extract_words,
    extract_sentences,
    extract_urls,
    remove_punctuation,
    remove_digits,
)

def characters_count(text: str) -> int:
    return len(text)

def words_count(text: str) -> int:
    return len(extract_words(remove_punctuation(text)))

def unique_words_count(text: str) -> int:
    return len(set(extract_words(remove_punctuation(text))))

def sentences_count(text: str) -> int:
    return len(extract_sentences(text))

def urls_count(text: str) -> int:
    return len(extract_urls(text))

def characters_per_word(text: str) -> float:
    words = extract_words(
        remove_digits(remove_punctuation(text))
    )
    characters_cnt = 0
    for w in words:
        characters_cnt += len(w)
    words_cnt = len(words)
    return characters_cnt / words_cnt

def words_per_sentence(text: str) -> float:
    sentences_cnt = sentences_count(text)
    words_cnt = words_count(text)
    return words_cnt / sentences_cnt

def count_words_by_predicat(text, predicat_func: Callable) -> int:
    words = extract_words(
        remove_digits(remove_punctuation(text))
    )
    words_cnt = 0
    for w in words:
        if predicat_func(w):
            words_cnt += 1
    return words_cnt

def long_words_count(text: str, normalize: bool = False) -> Union[int, float]:
    LONG_WORD_LENGTH = 8
    long_words_cnt = count_words_by_predicat(
        text, predicat_func=lambda x: len(x) >= LONG_WORD_LENGTH
    )
    words_cnt = words_count(text)
    if normalize:
        return long_words_cnt / words_cnt
    else:
        return long_words_cnt

def short_words_count(text: str, normalize: bool = False) -> Union[int, float]:
    SHORT_WORD_LENGTH = 4
    short_words_cnt = count_words_by_predicat(
        text, predicat_func=lambda x: len(x) <= SHORT_WORD_LENGTH
    )
    words_cnt = words_count(text)
    if normalize:
        return short_words_cnt / words_cnt
    else:
        return short_words_cnt

def lexical_density(stopwords: List[str]):
    pass

def gunning_fog_index():
    pass

def coleman_liau_grade():
    pass

def flesch_kincaid_grade_level():
    pass

def smog_grade():
    pass

def laesbarhedsindex():
    pass






# -*- coding: utf-8 -*-

"""
Function for extracting information from texts.
"""

import re
import string
from typing import List
from razdel import sentenize
from nltk import tokenize
import textarium.preprocessing as preprocessing

def extract_words(text: str) -> List[str]:
    """Extract a list of tokens from a text

    Args:
        text (str): Any string

    Returns:
        List[str]: A list of extracted tokens
    """
    words = tokenize.word_tokenize(text)
    return words

def extract_sentences(text: str, lang='en') -> List[str]:
    """Extract a list of sentences from a text

    Args:
        text (str): Any string in English or Russian
        lang (str): Text language ('ru' - Russian or 'en' - English)

    Returns:
        List[str]: A list of extracted sentences
    """
    if lang == 'en':
        sentences = tokenize.sent_tokenize(text)
        sentences = [preprocessing.remove_extra_spaces(s) for s in sentences]
    elif lang == 'ru':
        sentences = [i.text for i in sentenize(text)]
    return sentences

def extract_urls(text:str) -> List[str]:
    """Extract a list of URLs from a text

    Args:
        text (str):Any string

    Returns:
        List[str]: A list of extracted URLs
    """
    url_regex = re.compile(
        "((https?):((//)|(\\\\))+([\w\d:#@%/;$()~_?\+-=\\\.&](#!)?)*)", re.DOTALL
    )
    parsed_url_objects = re.findall(url_regex, text)
    urls = [url_param[0].strip(string.punctuation) for url_param in parsed_url_objects]
    return urls



from unittest import TestCase
from textarium.collections import get_stopwords

class TestCollections(TestCase):
    def test_get_stopwords_en_0(self):
        sw = get_stopwords()
        self.assertTrue(len(sw) > 0)
        self.assertTrue(type(sw) == list)

    def test_get_stopwords_ru_0(self):
        sw = get_stopwords(lang='ru')
        self.assertTrue(len(sw) > 0)
        self.assertTrue(type(sw) == list)


from unittest import TestCase

import textarium.extraction as extraction

class TestExtraction(TestCase):
    def test_extract_words_en_0(self):
        test_input = "This line has 5 tokens"
        expected_result = ["This", "line", "has", "5", "tokens"]
        result = extraction.extract_words(test_input)
        self.assertEqual(expected_result, result)

    def test_extract_words_ru_0(self):
        test_input = "В этой строке 5 токенов"
        expected_result = ["В", "этой", "строке", "5", "токенов"]
        result = extraction.extract_words(test_input)
        self.assertEqual(expected_result, result)

    def test_extract_sentences_en_0(self):
        text_input = """
        Hello! My name is Robbie. 
        Please, write an email to Mr. Parker. 
        His email: parker@gmail.com.
        """
        expected_result = [
            "Hello!", "My name is Robbie.", 
            "Please, write an email to Mr. Parker.", 
            "His email: parker@gmail.com."
        ]
        result = extraction.extract_sentences(text_input, lang='en')
        self.assertEqual(expected_result, result)

    def test_extract_sentences_ru_0(self):
        text_input = """
        Привет! Меня зовут Робби.
        Пожалуйста, напиши письмо (т. е. e-mail) Паркеру.
        Его адрес: parker@gmail.com.
        """
        expected_result = [
            "Привет!", "Меня зовут Робби.",
            "Пожалуйста, напиши письмо (т. е. e-mail) Паркеру.",
            "Его адрес: parker@gmail.com."
        ]
        result = extraction.extract_sentences(text_input, lang='ru')
        self.assertEqual(expected_result, result)

    def test_extract_urls_0(self):
        text_input = """
        There is one link: http://google.com.
        And another one: https://www.google.com/images?v=1&p=2!
        This is not a link: test@gmail.com
        """
        exptected_result = [
            "http://google.com",
            "https://www.google.com/images?v=1&p=2"
        ]
        result = extraction.extract_urls(text_input)
        self.assertEqual(exptected_result, result)



# -*- coding: utf-8 -*-

"""
Corpus class.
"""
from typing import List
from .text import Text

class Corpus:
    def __init__(self, texts: List[str], lang='en'):
        self.lang = lang
        self.corpus = [Text(t, lang=self.lang) for t in texts]

    def __str__(self) -> str:
        first_five_texts = "\n\n".join([t.raw_text for t in self.corpus[:5]])
        last_five_texts = "\n\n".join([t.raw_text for t in self.corpus[-5:]])
        return first_five_texts + "\n\n...\n\n" + last_five_texts

    def __repr__(self) -> str:
        return f"textarium.Corpus object containing {len(self.corpus)} textarium.Text objects"

    def prepare(self, lemmatize: bool = True, stopwords: List[str] = None):
        for t in self.corpus:
            t.prepare(lemmatize=lemmatize, stopwords=stopwords)


from unittest import TestCase
from textarium.corpus import Corpus

class TestText(TestCase):
    def test_prepare_en_0(self):
        input = [
            "Hello! My name is Mr.Parker.",
            "I have a website https://parker.com.",
            "It has about 5000 visitors per day.",
            "I track it with a simple html-block like this:",
            "<div>Google.Analytics</div>",
        ]
        expected_result = [
            "hello my name is mr parker",
            "i have a website",
            "it ha about visitor per day",
            "i track it with a simple html block like this",
            "google analytics",
        ]
        corpus = Corpus(input, lang='en')
        corpus.prepare()
        self.assertListEqual(expected_result, [t.prepared_text for t in corpus.corpus])



# -*- coding: utf-8 -*-

"""
Utility functions
"""

from functools import reduce
from typing import Any, Callable, Sequence

def pipeline(
        arg: Any,
        function_pipeline: Sequence[Callable],
) -> Any:
    """A generic Unix-like pipeline

    Args:
        arg (Any): The value you want to pass through a pipeline
        function_pipeline (Sequence[Callable]): An ordered 
        list of functions that comprise your pipeline

    Returns:
        Any: A result of functions pipeline
    """
    return reduce(lambda v, f: f(v), function_pipeline, arg)
